Imports System.IO
''' <summary>
''' Liste mit MP3-Abschnitten
''' </summary>
''' <remarks>speichert MP3-Datei-Sequenzen und schreibt den Inhalt einer MP3-Datei in einen Stream</remarks>
Public Class MP3ReceivingFile
    Inherits MP3File
    Implements IDisposable

#Region " Fields / Events "

    ''' <summary>
    ''' FileStream, in den die Daten der MP3-Datei geschrieben werden
    ''' </summary>
    Private m_FS As IO.FileStream

    ''' <summary>
    ''' Flag - True, wenn die Datei von der RadioStation mit weiteren Abschnitten versorgt wird
    ''' </summary>
    Private m_Receiving As Boolean = False

    ''' <summary>
    ''' Aktuelle Lnge der MP3-Datei
    ''' </summary>
    Private m_FileLength As Long = 0

    ''' <summary>
    ''' Aktuell bearbeiteter Abschnitt der MP3-Datei
    ''' </summary>
    Private m_AcSection As MP3FileSection

    ''' <summary>
    ''' Aktueller Musiktitel
    ''' </summary>
    Private m_CurrentTitle As String = ""

    ''' <summary>
    ''' Event, das geliefert wird, wenn sich der aktuelle Musiktitel ndert
    ''' </summary>
    Public Event CurrentTitle_Changed()

    ''' <summary>
    ''' Timer zum Liefern von nderungs-Events beim Aufzeichnen
    ''' </summary>
    Private WithEvents m_Timer As Timer

    ''' <summary>
    ''' Zuletzt empfangener Text mit Titel und Interpret
    ''' </summary>
    Private m_LastText As String = ""

    ''' <summary>
    ''' Flag: Information, ob das Bearbeiten der MP3-Datei ordnungsgemaess beendet wurde
    ''' </summary>
    Private m_Closed As Boolean = False

    ''' <summary>
    ''' Info, ob Dispose bereits einmal ausgefhrt wurde
    ''' </summary>
    Private m_disposed As Boolean = False

#End Region

#Region " Constructor "

    ''' <summary>
    ''' Konstruktor zum Erstellen einer neuen MP3-Dateibersicht 
    ''' </summary>
    ''' <param name="MP3FileName">Datei-Name</param>
    ''' <param name="List">Parent-Liste</param>
    ''' <param name="StationName">Name der Radio-Station</param>
    Public Sub New(ByVal MP3FileName As String, ByVal List As MP3Source, ByVal StationName As String)
        MyBase.New(MP3FileName, List, StationName)
        m_Receiving = True
        m_Timer = New Timer
        m_Timer.Interval = 1000
        m_Timer.Start()
        m_FS = New FileStream(MP3Source.OutputPath & "\" & MP3FileName & ".mp3", FileMode.OpenOrCreate, FileAccess.Write, FileShare.Read)
    End Sub

#End Region

#Region " Properties "

    ''' <summary>
    ''' Info, ob die Datei von der RadioStation mit weiteren Abschnitten versorgt wird
    ''' </summary>
    Public Overrides ReadOnly Property Receiving() As Boolean
        Get
            Return m_Receiving
        End Get
    End Property

    ''' <summary>
    ''' Aktuelle Lnge der MP3-Datei
    ''' </summary>
    ReadOnly Property FileLenght() As Long
        Get
            Return m_FileLength
        End Get
    End Property

    ''' <summary>
    ''' Aktueller Musiktitel und Interpret
    ''' </summary>
    Property CurrentTitle() As String
        Get
            Return m_CurrentTitle
        End Get
        Set(ByVal value As String)
            If m_CurrentTitle <> value Then
                If value = "" OrElse m_LastText = value Then Exit Property
                m_LastText = value

                Dim TmpText As String = Clean_Text(value)

                If TmpText <> "" AndAlso m_CurrentTitle <> TmpText Then
                    m_CurrentTitle = TmpText

                    If m_AcSection Is Nothing Then
                        m_AcSection = New MP3FileSection(m_CurrentTitle, m_FileLength, Now.AddSeconds(-1))
                        Me.Add(m_AcSection)
                    Else
                        m_AcSection.EndPosition = m_FileLength
                        m_AcSection = New MP3FileSection(m_CurrentTitle, m_FileLength, Now)
                        Me.Add(m_AcSection)
                    End If
                    RaiseEvent CurrentTitle_Changed()
                End If
            End If
        End Set
    End Property


#End Region

#Region " Methods "

    ''' <summary>
    ''' bereinigt einen String von doppelten Leerzeichen oder Anfuehrungszeichen
    ''' </summary>
    Private Function Clean_Text(ByVal Source As String) As String
        Dim TmpText As String = Source.Replace(Chr(34), " ")
        TmpText = TmpText.Replace("_", " ").Trim
        If TmpText.StartsWith("-") Then
            TmpText = TmpText.Substring(1).Trim
        End If
        If TmpText.EndsWith("-") Then
            TmpText = TmpText.Substring(0, TmpText.Length - 1).Trim
        End If

        Dim Idx As Integer = TmpText.IndexOf("  ")
        While Idx > -1
            TmpText = TmpText.Replace("  ", " ")
            Idx = TmpText.IndexOf("  ")
        End While

        Return TmpText
    End Function

    ''' <summary>
    ''' fgt ein Array von Bytes an das Ende des FileStreams der MP3-Datei an
    ''' </summary>
    ''' <param name="array">Byte-Array</param>
    ''' <param name="count">Anzahl der zu schreibenden Bytes</param>
    Public Sub Write(ByVal array As Byte(), ByVal count As Integer)
        m_FS.Write(array, 0, count)
        m_FileLength += count
    End Sub

    ''' <summary>
    ''' Schliesst die Datei und den ggf. noch geoeffneten FileStream
    ''' </summary>
    Public Sub CloseFile()
        If m_Closed Then Exit Sub
        m_Closed = True
        m_FS.Close()

        If m_FS IsNot Nothing Then
            m_FS.Dispose()
        End If

        m_Timer.Stop()
        RecordingEnd = Now
        m_Receiving = False

        If m_FileLength = 0 Then
            MyBase.Delete()
        Else
            If m_AcSection IsNot Nothing Then
                m_AcSection.EndPosition = m_FileLength
            Else
                m_AcSection = New MP3FileSection(StationName, m_FileLength, Now)
                Me.Add(m_AcSection)
            End If
            MyBase.RaiseReceivingChanged()
        End If
    End Sub

    ''' <summary>
    ''' ndert die Anzeige im TreeNode abhngig von der Aufzeichnungs-Dauer
    ''' </summary>
    Private Sub m_Timer_Tick(ByVal sender As Object, ByVal e As System.EventArgs) Handles m_Timer.Tick
        RecordingEnd = Now
    End Sub

    Protected Overridable Sub Dispose(ByVal disposing As Boolean)
        If Not Me.m_disposed Then
            If disposing Then
                ' TODO: Verwaltete Ressourcen freigeben, wenn sie explizit aufgerufen werden
                CloseFile()
            End If

            ' TODO: Gemeinsam genutzte nicht verwaltete Ressourcen freigeben
        End If
        Me.m_disposed = True
    End Sub

    ' Dieser Code wird von Visual Basic hinzugefgt, um das Dispose-Muster richtig zu implementieren.
    Public Sub Dispose() Implements IDisposable.Dispose
        ' ndern Sie diesen Code nicht. Fgen Sie oben in Dispose(ByVal disposing As Boolean) Bereinigungscode ein.
        Dispose(True)
        GC.SuppressFinalize(Me)
    End Sub

#End Region

End Class

