Imports System.Text

''' <summary>
''' Klasse zum Sammeln von Bytes und anderer Daten beim Empfang eines MP3-Streams
''' </summary>
''' <remarks></remarks>
Public Class ReceivedByteList
    Inherits List(Of Byte)

#Region " Fields / Events "

    ''' <summary>
    ''' Meta-Intervall des Shoutcast-Streams
    ''' </summary>
    Private m_MetaInterval As Integer

    ''' <summary>
    ''' Bitrate des Shoutcast-Streams
    ''' </summary>
    Private m_BitRate As Integer

    ''' <summary>
    ''' Radiostations-Name des Shoutcast-Streams
    ''' </summary>
    Private m_StationName As String = ""

    ''' <summary>
    ''' Flag zum Beenden des Empfangs
    ''' </summary>
    Private m_IsValid As Boolean = True

    ''' <summary>
    ''' Event bei nderung des Radiostations-Namens
    ''' </summary>
    Public Event StationName_Changed()

    ''' <summary>
    ''' Event, wenn mehr Bytes vorhanden sind, als fr ein Meta-Intervall erforderlich
    ''' </summary>
    Public Event HaveDataToWrite()

#End Region

#Region " Constructor "

    ''' <summary>
    ''' Default Konstruktor
    ''' </summary>
    Public Sub New(ByVal StationName As String)
        m_StationName = StationName
    End Sub

#End Region

#Region " Properties "

    ''' <summary>
    ''' Radiostations-Name des Shoutcast-Streams
    ''' </summary>
    Property StationName() As String
        Get
            Return m_StationName
        End Get
        Set(ByVal value As String)
            Dim NewValue As String = value.Trim
            If NewValue <> "" AndAlso m_StationName <> NewValue Then
                m_StationName = NewValue
                RaiseEvent StationName_Changed()
            End If
        End Set
    End Property

    ''' <summary>
    ''' Meta-Intervall des Shoutcast-Streams
    ''' </summary>
    Property MetaInterval() As Integer
        Get
            Return m_MetaInterval
        End Get
        Set(ByVal value As Integer)
            m_MetaInterval = value
        End Set
    End Property

    ''' <summary>
    ''' Bitrate des Shoutcast-Streams
    ''' </summary>
    Property BitRate() As Integer
        Get
            Return m_BitRate
        End Get
        Set(ByVal value As Integer)
            m_BitRate = value
        End Set
    End Property

    ''' <summary>
    ''' Flag zum Beenden des Empfangs
    ''' </summary>
    Property IsValid() As Boolean
        Get
            Return m_IsValid
        End Get
        Set(ByVal value As Boolean)
            m_IsValid = value
        End Set
    End Property

#End Region

#Region " Methods "

    ''' <summary>
    ''' Fgt empfangene Bytes zur Liste hinzu und liefert, 
    ''' falls die Liste lnger als das Meta-Intervall ist, ein Event
    ''' </summary>
    Public Shadows Sub AddRange(ByVal collection As IEnumerable(Of Byte))
        MyBase.AddRange(collection)
        If Me.Count > m_MetaInterval Then
            RaiseEvent HaveDataToWrite()
        End If
    End Sub

    ''' <summary>
    ''' liefert eine Array aus Bytes mit Meta-Intervall-Lnge
    ''' </summary>
    Public Function GetMusicBytes() As Byte()
        Dim Bytes() As Byte

        Bytes = MyBase.GetRange(0, m_MetaInterval).ToArray
        MyBase.RemoveRange(0, m_MetaInterval)

        Return Bytes
    End Function


    ''' <summary>
    ''' liefert einen String Musiktitel und Interpret, falls entsprechende Bytes 
    ''' zwischen zwei Meta-Intervallen vorhanden sind
    ''' </summary>
    Public Function GetText() As String
        Dim Result As String = ""

        Dim Laenge As Integer = CInt(MyBase.Item(0)) * 16
        If Laenge = 0 Then
            MyBase.RemoveAt(0)
        Else
            Result = Encoding.ASCII.GetString(MyBase.GetRange(1, Laenge).ToArray).Trim
            MyBase.RemoveRange(0, Laenge + 1)
        End If

        Return Result
    End Function

#End Region

End Class

