Imports System.Xml
''' <summary>
''' eine einmalige Aufzeichnungs-Aufgabe mit Start und Ende
''' </summary>
''' <remarks></remarks>
Public Class RecordingTaskSingle
    Inherits RecordingTask

#Region " Fields / Events "

    ''' <summary>
    ''' Start-Zeitpunkt
    ''' </summary>
    Private m_StartTime As Date

    ''' <summary>
    ''' End-Zeitpunkt
    ''' </summary>
    Private m_EndTime As Date

#End Region

#Region " Constructor "

    ''' <summary>
    ''' Standard-Konstruktor
    ''' </summary>
    Public Sub New()
        m_StartTime = Now
        m_EndTime = m_StartTime.AddHours(1)
    End Sub

    ''' <summary>
    ''' Konstruktor mit Start und Ende
    ''' </summary>
    Public Sub New(ByVal StartTime As Date, ByVal EndTime As Date)
        m_StartTime = StartTime
        m_EndTime = EndTime
    End Sub

    ''' <summary>
    ''' Konstruktor zum Erstellen beim Lesen einer XML-Datei
    ''' </summary>
    Public Sub New(ByVal Nd As XmlNode)
        m_StartTime = New DateTime(CLng(Nd.Attributes("StartTime").Value))
        m_EndTime = New DateTime(CLng(Nd.Attributes("EndTime").Value))
    End Sub

#End Region

#Region " Properties "

    ''' <summary>
    ''' Start-Zeitpunkt
    ''' </summary>
    Property StartTime() As Date
        Get
            Return m_StartTime
        End Get
        Set(ByVal value As Date)
            If m_StartTime <> value Then
                m_StartTime = value
                If m_StartTime >= m_EndTime Then
                    m_EndTime = m_StartTime.AddHours(1)
                End If

                MyBase.RaiseChanged()
            End If
        End Set
    End Property

    ''' <summary>
    ''' End-Zeitpunkt
    ''' </summary>
    Property EndTime() As Date
        Get
            Return m_EndTime
        End Get
        Set(ByVal value As Date)
            If m_EndTime <> value Then
                m_EndTime = value
                If m_EndTime <= m_StartTime Then
                    m_StartTime = m_EndTime.AddHours(-1)
                End If

                MyBase.RaiseChanged()
            End If
        End Set
    End Property

    ''' <summary>
    ''' zum Identifizieren ohne Reflection
    ''' </summary>
    Public Overrides ReadOnly Property Type() As RecordingTaskTypes
        Get
            Return RecordingTaskTypes.SingleTask
        End Get
    End Property
#End Region

#Region " Methods "

    ''' <summary>
    ''' schreibt den Inhalt in eine XML-Datei
    ''' </summary>
    Public Overrides Sub Write_XML(ByVal Writer As XmlTextWriter)
        Writer.WriteStartElement("SingleTask")
        Writer.WriteAttributeString("StartTime", m_StartTime.Ticks.ToString)
        Writer.WriteAttributeString("EndTime", m_EndTime.Ticks.ToString)
        Writer.WriteEndElement()
    End Sub

    ''' <summary>
    ''' gibt zurck, ob diese Aufgabe zum angegebenen Zeitpunkt aktiv ist
    ''' </summary>
    Public Overrides Function IsActive(ByVal Time As Date) As Boolean
        Dim TmpStart As DateTime = m_StartTime.AddSeconds(-3)
        Dim TmpEnde As DateTime = m_EndTime.AddSeconds(3)
        If TmpStart <= Time AndAlso TmpEnde >= Time Then
            Return True
        End If
        Return False
    End Function

    ''' <summary>
    ''' erstellt eine neue RecordingTask-Instanz mit gleichem Inhalt
    ''' </summary>
    Public Overrides Function CreateClone() As RecordingTask
        Return New RecordingTaskSingle(m_StartTime, m_EndTime)
    End Function

    ''' <summary>
    ''' liefert ein Control zum Bearbeiten der aktuellen RecordingTask-Instanz
    ''' </summary>
    Public Overrides Function EditControl() As System.Windows.Forms.Control
        Return New CtlSingleRecorderTask(Me)
    End Function

    ''' <summary>
    ''' liefert einen String zum Anzeigen in Controls
    ''' </summary>
    Public Overrides Function ToString() As String
        Dim TS As New TimeSpan(m_EndTime.Ticks - m_StartTime.Ticks)
        Return "Start: " & m_StartTime.ToShortDateString & " " & m_StartTime.ToShortTimeString & ", " & My.LanguageInfo.TextTimeLength & ": " & SharedFunctions.TimeSpanStringMinutes(TS)
    End Function

#End Region

End Class

