Imports System.Drawing.Drawing2D
Imports System.ComponentModel
''' <summary>
''' Ein Label mit farbigem Hintergrund-Gradienten
''' </summary>
Public Class ColorLabel
    Inherits System.Windows.Forms.Label

#Region " Fields / Events "
    ' Bitmap, auf der gezeichnet und deren Inhalt angezeigt wird
    Private m_BMP As Bitmap = Nothing
    ' Textfarbe
    Private m_TextColor As Color = Color.White
#End Region

#Region " Constructor "
    Public Sub New()
        Me.BackColor = System.Drawing.Color.RoyalBlue
        Me.ForeColor = System.Drawing.Color.White
        Me.Size = New System.Drawing.Size(100, 16)
        Me.TextAlign = System.Drawing.ContentAlignment.MiddleLeft
        ' DoubleBuffering aktievieren
        Me.SetStyle(ControlStyles.DoubleBuffer Or ControlStyles.UserPaint _
        Or ControlStyles.AllPaintingInWmPaint, True)
        MyBase.AutoSize = False
    End Sub
#End Region

#Region " Properties "
    <Description("Legt die Farbe fr den angezeigten Text fest oder gibt diese zurck.")> _
      Property TextColor() As Color
        Get
            Return m_TextColor
        End Get
        Set(ByVal value As Color)
            m_TextColor = value
            m_BMP = Nothing
            Invalidate()
        End Set
    End Property
#End Region

#Region " Methods "
    ''' <summary>
    ''' berschreiben von OnPaint, um den Inhalt des Controls zu zeichnen
    ''' </summary>
    Protected Overrides Sub OnPaint(ByVal pe As System.Windows.Forms.PaintEventArgs)
        Dim Rect As New Rectangle(0, 0, Me.Width, Me.Height)

        If m_BMP Is Nothing Then
            m_BMP = New Bitmap(Me.Width, Me.Height)
            Using g As Graphics = Graphics.FromImage(m_BMP)
                If Me.Enabled Then
                    Dim BackBrush As New LinearGradientBrush(Rect, Me.ForeColor, Me.BackColor, 115.0F)
                    g.FillRectangle(BackBrush, Rect)
                    Using ForeBrush As New SolidBrush(m_TextColor)
                        g.DrawString(Me.Text, Me.Font, ForeBrush, 2, 2)
                    End Using
                Else
                    Dim BackBrush As New LinearGradientBrush(Rect, Color.White, Color.LightGray, 115.0F)
                    g.FillRectangle(BackBrush, Rect)
                    Using ForeBrush As New SolidBrush(Color.DarkGray)
                        g.DrawString(Me.Text, Me.Font, ForeBrush, 2, 2)
                    End Using
                End If
            End Using
        End If
        pe.Graphics.DrawImage(m_BMP, Rect)
    End Sub

    ''' <summary>
    ''' berschreiben von OnResize, um den Inhalt des Controls neu zu zeichnen
    ''' </summary>
    Protected Overrides Sub OnResize(ByVal e As System.EventArgs)
        MyBase.OnResize(e)
        m_BMP = Nothing
        Invalidate()
    End Sub
    ''' <summary>
    ''' berschreiben von OnBackColorChanged, um den Inhalt des Controls neu zu zeichnen
    ''' </summary>
    Protected Overrides Sub OnBackColorChanged(ByVal e As System.EventArgs)
        MyBase.OnBackColorChanged(e)
        m_BMP = Nothing
        Invalidate()
    End Sub
    ''' <summary>
    ''' berschreiben von OnForeColorChanged, um den Inhalt des Controls neu zu zeichnen
    ''' </summary>
    Protected Overrides Sub OnForeColorChanged(ByVal e As System.EventArgs)
        MyBase.OnForeColorChanged(e)
        m_BMP = Nothing
        Invalidate()
    End Sub
    ''' <summary>
    ''' berschreiben von OnTextChanged, um den Inhalt des Controls neu zu zeichnen
    ''' </summary>
    Protected Overrides Sub OnTextChanged(ByVal e As System.EventArgs)
        MyBase.OnTextChanged(e)
        m_BMP = Nothing
        Invalidate()
    End Sub
    ''' <summary>
    ''' berschreiben von OnEnabledChanged, um den Inhalt des Controls neu zu zeichnen
    ''' </summary>
    Protected Overrides Sub OnEnabledChanged(ByVal e As System.EventArgs)
        MyBase.OnEnabledChanged(e)
        m_BMP = Nothing
        Invalidate()
    End Sub
#End Region

End Class
