''' <summary>
''' Windows Form zum Speichern einer MP3-Datei
''' </summary>
''' <remarks></remarks>
Public Class frm_SaveMP3

#Region " Fields / Events "

    ''' <summary>
    ''' bevorzugte Optionen zum Speichern
    ''' </summary>
    Private m_Options As MP3SaveOptions

    ''' <summary>
    ''' Referenz zur MP3-Datei
    ''' </summary>
    Private m_File As MP3File

    ''' <summary>
    ''' Flag beim Initialisieren der angezeigten Werte
    ''' </summary>
    Private m_Init As Boolean = False

#End Region

#Region " Constructor "

    ''' <summary>
    ''' Konstruktor fr existierende MP3-Datei und Benutzer-Optionen
    ''' </summary>
    ''' <param name="Options">bevorzugte Optionen zum Speichern</param>
    ''' <param name="File">MP3-Datei</param>
    Public Sub New(ByVal Options As MP3SaveOptions, ByVal File As MP3File)

        ' Dieser Aufruf ist fr den Windows Form-Designer erforderlich.
        InitializeComponent()

        ' Fgen Sie Initialisierungen nach dem InitializeComponent()-Aufruf hinzu.
        m_Options = Options
        m_File = File

        ' Beschriftungen der Controls an aktuelle Sprache anpassen
        Me.ButtonCancel.Text = My.LanguageInfo.TextCancel
        Me.RadioButtonMultipleFiles.Text = My.LanguageInfo.TextCreateMultipleFiles
        Me.RadioButtonSingleFile.Text = My.LanguageInfo.TextCreateSingleFile
        Me.CheckBoxAddID3Tag.Text = My.LanguageInfo.TextAddID3Tag
        Me.CheckBoxEnumerateFiles.Text = My.LanguageInfo.TextEnumerateFiles
        Me.Text = My.LanguageInfo.TextSaveMP3

        Dim ValidSections As List(Of MP3FileSection) = GetSections(m_File)

        m_Init = True

        Me.CheckBoxEnumerateFiles.Checked = m_Options.EnumerateFiles
        Me.CheckBoxAddID3Tag.Checked = m_Options.AddID3Tag

        If m_Options.SaveMultipleFilesPreferred AndAlso ValidSections.Count > 1 Then
            Me.RadioButtonMultipleFiles.Checked = True
            Panel1.Visible = True
        Else
            Me.RadioButtonSingleFile.Checked = True
            Panel1.Visible = False
        End If

        If ValidSections.Count < 2 Then
            Me.RadioButtonMultipleFiles.Enabled = False
        End If

        m_Init = False
    End Sub

#End Region

#Region " Methods "

    ''' <summary>
    ''' liefert die Abschnitte einer MP3-Datei
    ''' </summary>
    Private Function GetSections(ByVal File As MP3File) As List(Of MP3FileSection)
        Dim Result As New List(Of MP3FileSection)
        Dim StartPosition As Long = 0
        Dim EndPosition As Long = 0

        For Each Sec As MP3FileSection In File
            EndPosition = Sec.EndPosition
            Dim Length As Long = EndPosition - StartPosition
            If Length > 0L Then
                Result.Add(Sec)
            End If
            StartPosition = EndPosition
        Next

        Return Result
    End Function

    ''' <summary>
    ''' tritt ein bei CheckedChanged-Event im RadioButton zum Aufteilen der MP3-Datei in einzelne Dateien
    ''' </summary>
    Private Sub RadioButtonMultipleFiles_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles RadioButtonMultipleFiles.CheckedChanged
        If m_Init Then Exit Sub
        If RadioButtonMultipleFiles.Checked Then
            Panel1.Visible = True
            m_Options.SaveMultipleFilesPreferred = True
        End If
    End Sub

    ''' <summary>
    ''' tritt ein bei CheckedChanged-Event im RadioButton zum Speichern als zusammenhngende MP3-Datei
    ''' </summary>
    Private Sub RadioButtonSingleFile_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles RadioButtonSingleFile.CheckedChanged
        If m_Init Then Exit Sub
        If RadioButtonSingleFile.Checked Then
            Panel1.Visible = False
            m_Options.SaveMultipleFilesPreferred = False
        End If
    End Sub

    ''' <summary>
    ''' tritt ein bei CheckedChanged-Event der CheckBox fr numerierte Dateinamen
    ''' </summary>
    Private Sub CheckBoxEnumerateFiles_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles CheckBoxEnumerateFiles.CheckedChanged
        If m_Init Then Exit Sub
        m_Options.EnumerateFiles = CheckBoxEnumerateFiles.Checked
    End Sub

    ''' <summary>
    ''' tritt ein bei CheckedChanged-Event der CheckBox fr das Hinzufgen von ID3-Tags
    ''' </summary>
    Private Sub CheckBoxAddID3Tag_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles CheckBoxAddID3Tag.CheckedChanged
        If m_Init Then Exit Sub
        m_Options.AddID3Tag = CheckBoxAddID3Tag.Checked
    End Sub

    ''' <summary>
    ''' fhrt zum Speichern-Dialog abhngig von der MultipleFiles-Auswahl
    ''' </summary>
    Private Function SaveSuccess() As Boolean
        If RadioButtonMultipleFiles.Checked Then
            Return Save_Multiple_Success()
        Else
            Return Save_Single_Success()
        End If
    End Function

    ''' <summary>
    ''' tritt ein bei Click auf den OK-Button
    ''' </summary>
    Private Sub ButtonOK_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ButtonOK.Click
        If SaveSuccess() Then
            Me.Close()
        End If
    End Sub


    ''' <summary>
    ''' Speichern als separate Einzeldateien
    ''' </summary>
    ''' <returns>True, wenn Speichern erfolgreich</returns>
    Private Function Save_Multiple_Success() As Boolean

        Dim Path As String = GetEmptyDirectory(m_Options.LastSavedDirectory)
        If Path = "" Then Return False

        Me.Enabled = False
        Dim progressform As New frm_Progress
        progressform.Show(Me)

        Dim Result As Boolean = True

        Try
            Dim Saver As New MP3FileSaverMultiple(m_File, Path, progressform, m_Options.EnumerateFiles, m_Options.AddID3Tag)
            Dim SaveThread As Threading.Thread
            SaveThread = New Threading.Thread(AddressOf Saver.Execute)
            SaveThread.Start()

            Do While SaveThread.IsAlive
                Application.DoEvents()
            Loop

            m_Options.LastSavedDirectory = Path
        Catch ex As Exception
            MsgBox(ex.Message)
            Return False
        Finally
            If progressform.CancelButtonPushed Then
                Result = False
            End If
            progressform.Close()
            Me.Enabled = True
            Me.Activate()
        End Try
        Return Result

    End Function

    ''' <summary>
    ''' Speichern als zusammenhngende MP3-Datei
    ''' </summary>
    ''' <returns>True, wenn Speichern erfolgreich</returns>
    Private Function Save_Single_Success() As Boolean
        ' ein einfacher SaveFileDialog, unter dem angegebenen Namen wird die Original-Datei gespeichert
        Dim Dlg As New SaveFileDialog
        Dlg.AddExtension = True
        Dlg.OverwritePrompt = True
        Dlg.Filter = "mp3 File (*.mp3)|*.mp3"
        Dlg.CheckPathExists = True
        If m_Options.LastSingleFileDirectory <> "" Then
            If IO.Directory.Exists(m_Options.LastSingleFileDirectory) Then
                Dlg.InitialDirectory = m_Options.LastSingleFileDirectory
            End If

        End If
        Dlg.FileName = m_File.FileName & ".mp3"

        If Dlg.ShowDialog <> DialogResult.OK Then
            Return False
        End If

        Me.Enabled = False
        Dim progressform As New frm_Progress
        progressform.Show(Me)

        Dim Result As Boolean = True

        Try
            Dim Saver As New MP3FileSaverSingle(m_File, Dlg.FileName, progressform)
            Dim SaveThread As Threading.Thread
            SaveThread = New Threading.Thread(AddressOf Saver.Execute)
            SaveThread.Start()

            Do While SaveThread.IsAlive
                Application.DoEvents()
            Loop

            Dim Directorypath As String = IO.Path.GetDirectoryName(Dlg.FileName)
            m_Options.LastSavedDirectory = Directorypath
            Process.Start(Directorypath)
        Catch ex As Exception
            MsgBox(ex.Message)
            Return False
        Finally
            If progressform.CancelButtonPushed Then
                Result = False
            End If
            progressform.Close()
            Me.Enabled = True
            Me.Activate()
        End Try
        Return Result

    End Function

    ''' <summary>
    ''' Zeigt einen Dialog zur Directory-Auswahl an und garantiert, dass nur ein Pfad fr ein leeres Directory 
    ''' zurck gegeben wird
    ''' </summary>
    Private Function GetEmptyDirectory(ByVal DefaultDirectory As String) As String
        If DefaultDirectory <> "" AndAlso Not IO.Directory.Exists(DefaultDirectory) Then
            DefaultDirectory = ""
        End If
        ' FolderBrowserDialog instanziieren
        Using Dlg As New FolderBrowserDialog
            If DefaultDirectory <> "" Then
                Dlg.SelectedPath = DefaultDirectory
            End If
            If Dlg.ShowDialog = DialogResult.OK Then
                ' wenn im Dialog OK ausgewhlt wurde
                If IO.Directory.GetDirectories(Dlg.SelectedPath).Length > 0 Then
                    Dim Result As DialogResult = MessageBox.Show("Der angegebene Ordner ist nicht leer." & vbCrLf & "Bitte whlen Sie einen leeren Ordner aus.", "Shoutcast Recorder", MessageBoxButtons.RetryCancel, MessageBoxIcon.Information)
                    If Result = DialogResult.Retry Then
                        ' Diese Funktion rekursiv aufrufen
                        Return Me.GetEmptyDirectory(DefaultDirectory)
                    Else
                        ' wenn abgebrochen, leeren Pfad zurck geben
                        Return ""
                    End If
                ElseIf IO.Directory.GetFiles(Dlg.SelectedPath).Length > 0 Then
                    Dim Result As DialogResult = MessageBox.Show("Der angegebene Ordner enthlt Dateien." & vbCrLf & "Sollen diese gelscht werden?", "Shoutcast Recorder", MessageBoxButtons.YesNo, MessageBoxIcon.Question)
                    If Result = DialogResult.Yes Then
                        Try
                            Dim FileNames() As String = IO.Directory.GetFiles(Dlg.SelectedPath)
                            For Each txt As String In FileNames
                                IO.File.Delete(txt)
                            Next
                            Return Dlg.SelectedPath
                        Catch ex As Exception
                            Return ""
                        End Try
                    Else
                        ' wenn abgebrochen, leeren Pfad zurck geben
                        Return ""
                    End If
                Else
                    Return Dlg.SelectedPath
                End If
            Else
                ' wenn abgebrochen, leeren Pfad zurck geben
                Return ""
            End If
        End Using
        ' das hier ist eigentlich berflssig...
        Return ""
    End Function

#End Region

End Class